#! /bin/sh
#

#
# /* Shell Script as part of zx8 (ps) file and folder encryption tool */
# 
# /*
# Copyright (c) 2012, Karl-Uwe Frank
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
#   1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# 
#   2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# #** zx8 (ps) algorithm developed by Karl-Uwe Frank
# */


# -----------------------------------------------
#
#  If a segement is broken
#  =================================
#
#  bzip2recover ARCHIVE.tbz
# 
#  bunzip2 rec*.bz2
# 
#  rec00033ARCHIVE.tbz.bz2
#  
#  bunzip2 rec*.bz2
#  
#  cat rec*ARCHIVE.tbz > ARCHIVE-revovered.tbz
# 
#  rm -f rec*ARCHIVE.tbz
# 
#  tar -xvjf ARCHIVE-revovered.tbz
# 
# -----------------------------------------------

trap 'exit 1' 15

trap bashtrap INT

bashtrap() {
	echo 
	echo "Program canceled!"
	echo

	reset
	
	exit 1
}

whiletrap() {
	exit 1
}


function ShowUsage() {
	echo
	echo "ENcryption:"
	echo $0" -e /Path/to/Source-Folder/  File-Name   SegmentSize(in MB)"
	echo
	echo "DEcryption:"
	echo $0" -d File.tbz.zx8  /Path/to/Target-Folder/"
	echo
	exit 1
}


function AskExtract() {		
	echo
	echo
	echo
	echo "#########################################################"
	echo
	echo "Procceed with 'p' to unpack the archive "
	echo "or cancle with 'c'"
	echo 
	echo "#########################################################"
	
	while read KEYSTORKE; do
	
		if [ "$KEYSTORKE" == "c" ]; then
			echo
			echo "CANCLED !!!!"
			echo
	
			exit 0
		fi
	
		if [ "$KEYSTORKE" == "p" ]; then
			break
		fi
	
	done
}


PYTHON="$(which python)"
PYPY="$(which pypy)"
MACOSX="$(uname -ar | grep -i darwin)"

if [ "x$PYTHON" == "x" ] && [ "x$PYPY" == "x" ]; then
	clear
	echo
	echo "No installation of Python or PyPy found!"
	echo
	exit 1
fi

if [ "$#" -lt "3" ]; then
	ShowUsage
fi

ZX8_PY="./zx8_ftCrypto.py"

# COMPRESS="tar --one-TARGET-system -cmpvf"
# EXTRACT="tar -xvf"
# EXT="tar"

# COMPRESS="tar --one-TARGET-system -cmpvzf"
# EXTRACT="tar -xvzf"
# EXT="tgz"

#
# In cas of error it can be partly recovers using UnArchiver or bzip2recover
#
COMPRESS="tar --one-file-system -cmpvjf"
EXTRACT="tar -xvjf"
EXT="tbz"



ACTION=${1#*-}
SOURCE="$2"
TARGET="$3"
MAXSIZE=$4

if [ "x$ACTION" != "xe" ] && [ "x$ACTION" != "xd" ]; then
	ShowUsage
fi

if [ "x$SOURCE" == "x" ]; then
	ShowUsage
fi



PASS1="x"
PASS2="y"

if [ "x$ACTION" == "xe" ]; then
	# ENcryption

	# until the two passphrases match and are not empty...
	until [ "$PASS1" = "$PASS2" -a -n "$PASS1" ]; do
					# the bash read buitlin has to support the -s option.
					# Don't use read without -s!!
					echo
					echo "***** EN-cryption *****"; echo
					read -s -p "Enter Passphrase: " PASS1; echo; sleep 1
					read -s -p "Re-enter Passphrase: " PASS2; echo
	done

	FOLDERSIZE=$(du -scm "$SOURCE" | tail -1 | cut -f1)
	
	if [ "x$MAXSIZE" == "x" ]; then
		MAXSIZE=2000 # 2000 MB default value
	fi

	# if lower than 
	if [ $FOLDERSIZE -lt $MAXSIZE ]; then
		# erstellt eine Datei	
		$COMPRESS "$TARGET.$EXT" "$SOURCE" 
	
		pypy $ZX8_PY -e "$TARGET.$EXT" "$TARGET.$EXT.zx8" "$PASS1"
		
		rm -f "$TARGET.$EXT"
	else
		BLOCKSIZE=$MAXSIZE"m"
		# Tar - compress a complete folder and spilt it later if necessary
		
		TARFILE="${TARGET##*/}""."$EXT"_"
		ARCHIVE="$TARGET""-split"
		mkdir "$ARCHIVE"
		
		if [ ! -d "$ARCHIVE" ]; then
			echo "Error creating ""$ARCHIVE"
			exit 1
		fi
				
		$COMPRESS - "$SOURCE" | split -a 3 -b $BLOCKSIZE  - "$ARCHIVE/$TARFILE"
		
		# rename the archive SOURCE to make sure
		# that we can check if all files a present
		LASTFILE=$(ls -1 "$ARCHIVE" | tail -1)
		TEMP="$TARGET""_aaa_""${LASTFILE##*_}"
		mv "$ARCHIVE" "$TEMP"
		ARCHIVE="$TEMP"
		
		### for i in $(ls -1 "$ARCHIVE"); do # does not work with space in file name
		ls -1 "$ARCHIVE" | while read i; do
			trap whiletrap INT
			pypy $ZX8_PY -e "$ARCHIVE/$i" "$ARCHIVE/$i.zx8" "$PASS1"
			rm -f "$ARCHIVE/$i"
		done
	fi
	
else
	# DEcryption
	echo "--- Decryption ---"
	read -s -p "Enter Passphrase: " PASS1; echo

	TARGET_FOLDER="$TARGET"
	
	if [ ! -d "$TARGET_FOLDER" ]; then
		mkdir "$TARGET_FOLDER"
	fi

	if [ -f "$SOURCE" ]; then
		# if this a single TAR File
		TARFILE="${SOURCE##*/}"
		TARFILE="${TARFILE%.zx8}"

		pypy $ZX8_PY -d "$SOURCE" "$TARGET_FOLDER/$TARFILE" "$PASS1"
		
		# wrong keyword?
		if [ "$?" == "1" ]; then
			exit 1
		fi

		# Shall the Archive be extracted now?
		AskExtract 
		
		$EXTRACT "$TARGET_FOLDER/$TARFILE" -C "$TARGET_FOLDER"
		rm -f "$TARGET_FOLDER/$TARFILE"
		
	else
		# Concatenate a complete folder and unpack it
		TARFILE="${TARGET##*/}"

		### for i in $(ls -1 "$SOURCE"); do # does not work with space in file name
		ls -1 "$SOURCE" | while read i; do
			trap whiletrap INT
			pypy $ZX8_PY -d "$SOURCE/$i" "$TARGET_FOLDER/${i%.zx8}" "$PASS1"
			
			# wrong keyword?
			if [ "$?" == "1" ]; then
				exit 1
			fi
			
			# Tar - Concatenate a complete folder and unpack it
			cat "$TARGET_FOLDER/${i%.zx8}" >> "$TARGET_FOLDER/$TARFILE.$EXT"
			
			rm -f "$TARGET_FOLDER/${i%.zx8}"
		done

		# Shall the Archive be extracted now?
		AskExtract 

		cd "$TARGET_FOLDER"
		
		$EXTRACT "$TARFILE.$EXT"
		rm -f "$TARFILE.$EXT"
	fi

fi


exit 0



