#! /bin/sh
#

#
# /* Shell Script as part of zx8 (ps) file and folder encryption tool */
# 
# /*
# Copyright (c) 2012, Karl-Uwe Frank
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
#   1. Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# 
#   2. Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# #** zx8 (ps) algorithm developed by Karl-Uwe Frank
# */

trap 'exit 1' 15


trap bashtrap INT

bashtrap() {
	rm -f /tmp/Files.lst

	echo 
	echo "Program canceled!"
	echo

	reset
	
	exit 1
}


function ShowUsage() {
	echo
	echo "ENcryption:"
	echo $0" -e /Path/to/Source/  /Path/to/Target/"
	echo
	echo "DEcryption:"
	echo $0" -d /Path/to/Source/  /Path/to/Target/"
	echo
	exit 1
}


PYTHON="$(which python)"
PYPY="$(which pypy)"
MACOSX="$(uname -ar | grep -i darwin)"

if [ "x$PYTHON" == "x" ] && [ "x$PYPY" == "x" ]; then
	clear
	echo
	echo "No installation of Python or PyPy found!"
	echo
	exit 1
fi

if [ "$#" -ne "3" ]; then
	ShowUsage
fi

ZX8_PY="./zx8_ftCrypto.py"



ACTION=${1#*-}
SOURCE="$2"
TARGET="$3"

if [ "x$ACTION" != "xe" ] && [ "x$ACTION" != "xd" ]; then
	ShowUsage
fi

if [ "x$SOURCE" == "x" ]; then
	ShowUsage
fi


# walk through the source folder and all subfolders
rm -f /tmp/Files.lst

find "$SOURCE" -name '*' > /tmp/Files.lst

SOURCE_FOLDER="${SOURCE##*/}"
TARGET="$TARGET/$SOURCE_FOLDER"

mkdir -p "$TARGET"

if [ ! -d "$TARGET" ]; then
	echo "Error creating ""$TARGET"
	exit 1
fi
				
if [ -f /tmp/Files.lst ]; then

	PASS1="x"
	PASS2="y"
	
	if [ "x$ACTION" == "xe" ]; then
		# ENcryption
	
		# until the two passphrases match and are not empty...
		until [ "$PASS1" = "$PASS2" -a -n "$PASS1" ]; do
						# the bash read buitlin has to support the -s option.
						# Don't use read without -s!!
						echo
						echo "***** EN-cryption *****"; echo
						read -s -p "Enter Passphrase: " PASS1; echo; sleep 1
						read -s -p "Re-enter Passphrase: " PASS2; echo
		done
	
		
		# read each line of the file
		while read LINE; do
			# is it a folder?
			if [ ! -f "$LINE" ]; then
				FOLDER="${LINE##*$SOURCE}"
				mkdir -p "$TARGET/$FOLDER"
			else
				FILE="${LINE##*/}"
				FOLDER="${LINE%%$FILE}"
				FOLDER="${FOLDER##*$SOURCE}"

				pypy $ZX8_PY -e "$LINE" "$TARGET$FOLDER$FILE.zx8" "$PASS1"
			fi
		done < /tmp/Files.lst

	else
		# DEcryption
		echo "--- Decryption ---"
		read -s -p "Enter Passphrase: " PASS1; echo
		 
		# read each line of the file
		while read LINE; do
			# is it a folder?
			if [ ! -f "$LINE" ]; then
				FOLDER="${LINE##*$SOURCE}"
				mkdir -p "$TARGET/$FOLDER"
			else
				FILE="${LINE##*/}"
				FOLDER="${LINE%%$FILE}"
				FOLDER="${FOLDER##*$SOURCE}"

				# only if file is encrypted with the proper extension
				if [ "x${FILE##*.}"	== "xzx8" ]; then
					FILE="${FILE%.zx8}"
				
					pypy $ZX8_PY -d "$LINE" "$TARGET$FOLDER$FILE" "$PASS1"
				
					# wrong keyword?
					if [ "$?" == "1" ]; then
						exit 1
					fi

				fi
			fi
		done < /tmp/Files.lst
		
	fi
	
fi


rm -f /tmp/Files.lst

exit 0



